#include <unistd.h>
#include <Fl/Fl.H>
#include <Fl/Fl_Window.H>
#include <Fl/Fl_Choice.H>
#include <Fl/Fl_Button.H>
#include <Fl/Fl_Box.H>
#include <Fl/Fl_Scroll.H>
#include <Fl/Fl_Group.H>
#include <Fl/Fl_Tabs.H>
#include <Fl/Fl_Check_Button.H>
#include <Fl/Fl_Input.H>
#include <Fl/Fl_Multiline_Output.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/fl_ask.H>

#include "fuseedit.hpp"
#include "dialogs.h"



UserInterface *ui;
Fl_Multiline_Output *output;
FILE *config;
char device[100];
fuseByte fuses[100];
int cfuseBytes,ypos;
Fl_Group *editor, *options;
group *root,*last;
shortnam *shortnames,*last_short;

#include "fct.cpp"

#ifdef WIN32
	#include "f_win.cpp"
#else
	#include "f_lin.cpp"
#endif

void cb_hexInput(Fl_Widget *w, void *vp)
{
	/* Aktualisierung von Hex aus */
	int byte, bit, wert;
	char st1[20];
	for(byte=0;byte<cfuseBytes;byte++){
		sscanf(fuses[byte].input->value(),"%x",&wert);
		for(bit=0;bit<8;bit++){
			fuses[byte].fuseBit[bit].btn->value(wert&1);
			//fuses[byte].fuseBit[bit].btn->redraw();
			wert>>=1;
		}
	}
	refreshOptions();
	w->parent()->redraw();
}


void cb_bitBtn(Fl_Widget *w, void *vp)
{
	/* Aktualisierung von den Bit-Buttons aus */
	int byte, bit, wert;
	char st1[20];
	for(byte=0;byte<cfuseBytes;byte++){
		wert=0;
		for(bit=0;bit<8;bit++){
			wert<<=1;
			wert+=fuses[byte].fuseBit[7-bit].btn->value()&1;
		}
		sprintf(st1,"%02X",wert);
		fuses[byte].input->value(st1);
	}
	refreshOptions();
	w->parent()->redraw();
}

void cb_default(Fl_Widget *w, void *vp)
{
	int byte, bit, wert;
	char st1[20];
	for(byte=0;byte<cfuseBytes;byte++){
		wert=0;
		for(bit=0;bit<7;bit++) {
			wert<<=1;
			wert+=fuses[byte].fuseBit[7-bit].defaultvalue;
		}
		wert=fuses[byte].defaultvalue;
		setByte(byte,wert);
	}
	refreshOptions();
	w->parent()->redraw();
}

void cb_expert(Fl_Widget *w, void *vp)
{
	int byte,bit;
	Fl_Check_Button *check;
	check=(Fl_Check_Button*) w;
	for(byte=0;byte<cfuseBytes;byte++){
#ifndef WIN32
		for(bit=0;bit<8;bit++){
			if(check->value()) fuses[byte].fuseBit[bit].btn->set_active();
			else if(fuses[byte].fuseBit[bit].mode) fuses[byte].fuseBit[bit].btn->clear_active();
		}
#endif
	}
	w->parent()->redraw();
}

void cb_option(Fl_Widget *w, void *vp)
{
	int byte, bit, wert;
	char st1[20];
	Fl_Choice *ch;
	group *gr;
	option *akt;
	rule *akt_r;
	int c;
	ch=(Fl_Choice*) w;
	gr=(group*) vp;
	c=ch->value();
	//fl_alert ("Gruppe %s geändert auf Auswahl %d",gr->name,c);
	akt=gr->first_option;
	while(c){
		if(!akt->next) return;
		akt=akt->next;
		c--;
	}
	akt_r=akt->first_rule;
	while(akt_r){
		//printf("ändere fuse %d, bit %d auf wert %d\n",akt_r->fuse,akt_r->bit,akt_r->value);
		fuses[akt_r->fuse].fuseBit[akt_r->bit].btn->value(akt_r->value);
		akt_r=akt_r->next;
	}
	for(byte=0;byte<cfuseBytes;byte++){
		wert=0;
		for(bit=0;bit<8;bit++){
			wert<<=1;
			wert+=fuses[byte].fuseBit[7-bit].btn->value()&1;
		}
		sprintf(st1,"%02X",wert);
		fuses[byte].input->value(st1);
	}
	cb_bitBtn(w,vp);
}

void cb_fuseEdit(Fl_Widget *w, void *vp)
{
	int b,c=-1,wert;
	char st1[2048],longnam[100],shortnam[50],title[200],*cp,*cp1;
	group *akt;
	Fl_Choice *ch;
	ch=(Fl_Choice*)vp;
	rewind(config);
	do{
		if(!nextNam(longnam,shortnam)) return;
		c++;
	} while(c!=ch->value());
	sprintf(title,"Fuses %s (%s)",longnam,shortnam);
	strcpy(device,shortnam);
	Fl_Window win(300,300,600,400,title);
	win.size_range(600,400);
	Fl_Button bt_default(5,5,80,20,"Defaults");
	bt_default.callback(cb_default);
	Fl_Button bt_read(85,5,80,20,"Read");
	bt_read.callback(cb_read);
	Fl_Button bt_write(165,5,80,20,"Write");
	bt_write.callback(cb_fuse_op,(void*)WRITE);
	Fl_Button bt_verify(245,5,80,20,"Verify");
	bt_verify.callback(cb_fuse_op,(void*)VERIFY);
#ifndef WIN32
	Fl_Check_Button bt_expert(485,5,80,20,"Expert");
	bt_expert.callback(cb_expert);
#endif
	Fl_Scroll scroll(0,30,600,370);
	//scroll.type(7);
	editor=new Fl_Group(0,30,580,1200);
	ypos=30;
	readConf();
	akt=root;
	while(akt){
		akt->drop=new Fl_Choice(160,ypos,400,20,akt->name);
		akt->drop->callback(cb_option,(void*) akt);
		option *akt_opt;
		akt_opt=akt->first_option;
		while(akt_opt){
			akt->drop->add(akt_opt->name);
			akt_opt=akt_opt->next;
		}
		ypos+=25;
		akt=akt->next;
	}
	for(c=0;c<cfuseBytes;c++){
		ypos+=10;
		fuses[c].input=new Fl_Input(100,ypos,50,20,fuses[c].name);
		ypos+=30;
		fuses[c].input->callback(cb_hexInput);
		wert=0;
		for(b=0;b<8;b++){
			fuses[c].fuseBit[b].box=new Fl_Box(5,ypos,50,20,fuses[c].fuseBit[b].name);
			fuses[c].fuseBit[b].box->align(FL_ALIGN_LEFT+FL_ALIGN_INSIDE);
			fuses[c].fuseBit[b].btn=new Fl_Check_Button(100,ypos,600,20,fuses[c].fuseBit[b].desc);
			ypos+=20;
			fuses[c].fuseBit[b].btn->value(fuses[c].fuseBit[b].defaultvalue);
			wert<<=1;
			wert+=fuses[c].fuseBit[7-b].defaultvalue;
			fuses[c].fuseBit[b].btn->callback(cb_bitBtn);
#ifndef WIN32
			if(fuses[c].fuseBit[b].mode) fuses[c].fuseBit[b].btn->clear_active();
#endif
		}
		fuses[c].defaultvalue=wert;
	}
	editor->end();
	scroll.end();
	win.end();
	win.set_modal();
	win.resizable(scroll);
	cb_bitBtn(w,0);
	win.show();
	while(win.visible()) Fl::wait();
	memset(fuses,0,sizeof(fuses));
	clean();
}

int main(int argc, char** argv)
{
	Fl_Scroll *scroll;
	char st1[2048],shortn[20];
	shortnam *new_short;
	int c,i=0;
	FL_NORMAL_SIZE = 11;
	ui=new UserInterface();
	ui->dude_location->value("avrdude");
	config=fopen("config.xml","r");
	rdConfigfile("config.txt");
	if(!config) return -1;
	ui->flash_format->add("auto");
	ui->flash_format->add("Intel Hex");
	ui->flash_format->add("raw");
	ui->eeprom_format->add("auto");
	ui->eeprom_format->add("Intel Hex");
	ui->eeprom_format->add("raw");
	init();
	ui->flash_format->value(0);
	ui->eeprom_format->value(0);
	for(c=1;c<argc;c++) rdConfig(argv[c]);
	readProgrammerList();
	ui->MainWin->show();
	c=0;
	while(nextNam(st1,shortn)){
	ui->device_type->add(st1);
	if(!strcmp(st1,device)) i=c;
	new_short=(shortnam*) calloc(sizeof(shortnam),1);
	strcpy(new_short->name,shortn);
	if(!shortnames) shortnames=new_short;
	else last_short->next=new_short;
	last_short=new_short;
	c++;
	}
	ui->device_type->value(i);
	ui->btn_fuses->callback(cb_fuseEdit,(void*)ui->device_type);
	ui->MainWin->begin();
	scroll=new Fl_Scroll(10,280,600,380);
	output=new Fl_Multiline_Output(10,280,580,800);
	scroll->end();
	ui->MainWin->end();
	Fl::run();
	fclose(config);
	wrConfigfile("config.txt");
	return 0;
}

#include "dialogs.cxx"
